@@ -3,16 +3,6 @@ from PyObjCTools.TestSupport import *
 from CoreText import *
 from Quartz import *
 
-try:
-    unicode
-except NameError:
-    unicode = str
-
-try:
-    long
-except NameError:
-    long = int
-
 class TestCTFont (TestCase):
     def testTypes(self):
         self.assertIsInstance(CTFontRef, objc.objc_class)
@@ -33,10 +23,37 @@ class TestCTFont (TestCase):
         self.assertIsInstance(kCTBaselineClassRoman, unicode)
         self.assertIsInstance(kCTBaselineOriginalFont, unicode)
         self.assertIsInstance(kCTBaselineReferenceFont, unicode)
-        self.assertIsInstance(kCTBaselineReferenceFont, unicode)
 
         self.assertEqual(kCTFontTableAnkr, fourcc(b'ankr'))
 
+        self.assertEqual(kCTFontUIFontNone, 0xffffffff)
+        self.assertEqual(kCTFontUIFontUser, 0)
+        self.assertEqual(kCTFontUIFontUserFixedPitch, 1)
+        self.assertEqual(kCTFontUIFontSystem, 2)
+        self.assertEqual(kCTFontUIFontEmphasizedSystem, 3)
+        self.assertEqual(kCTFontUIFontSmallSystem, 4)
+        self.assertEqual(kCTFontUIFontSmallEmphasizedSystem, 5)
+        self.assertEqual(kCTFontUIFontMiniSystem, 6)
+        self.assertEqual(kCTFontUIFontMiniEmphasizedSystem, 7)
+        self.assertEqual(kCTFontUIFontViews, 8)
+        self.assertEqual(kCTFontUIFontApplication, 9)
+        self.assertEqual(kCTFontUIFontLabel, 10)
+        self.assertEqual(kCTFontUIFontMenuTitle, 11)
+        self.assertEqual(kCTFontUIFontMenuItem, 12)
+        self.assertEqual(kCTFontUIFontMenuItemMark, 13)
+        self.assertEqual(kCTFontUIFontMenuItemCmdKey, 14)
+        self.assertEqual(kCTFontUIFontWindowTitle, 15)
+        self.assertEqual(kCTFontUIFontPushButton, 16)
+        self.assertEqual(kCTFontUIFontUtilityWindowTitle, 17)
+        self.assertEqual(kCTFontUIFontAlertHeader, 18)
+        self.assertEqual(kCTFontUIFontSystemDetail, 19)
+        self.assertEqual(kCTFontUIFontEmphasizedSystemDetail, 20)
+        self.assertEqual(kCTFontUIFontToolbar, 21)
+        self.assertEqual(kCTFontUIFontSmallToolbar, 22)
+        self.assertEqual(kCTFontUIFontMessage, 23)
+        self.assertEqual(kCTFontUIFontPalette, 24)
+        self.assertEqual(kCTFontUIFontToolTip, 25)
+        self.assertEqual(kCTFontUIFontControlContent, 26)
 
     def testConstants(self):
         self.assertIsInstance(kCTFontCopyrightNameKey, unicode)
@@ -58,17 +75,17 @@ class TestCTFont (TestCase):
         self.assertIsInstance(kCTFontSampleTextNameKey, unicode)
         self.assertIsInstance(kCTFontPostScriptCIDNameKey, unicode)
 
-        self.assertEqual(kCTFontNoFontType,  4294967295) # (uint32_t)-1
-        self.assertEqual(kCTFontUserFontType,  0)
-        self.assertEqual(kCTFontUserFixedPitchFontType,  1)
-        self.assertEqual(kCTFontSystemFontType,  2)
-        self.assertEqual(kCTFontEmphasizedSystemFontType,  3)
-        self.assertEqual(kCTFontSmallSystemFontType,  4)
-        self.assertEqual(kCTFontSmallEmphasizedSystemFontType,  5)
-        self.assertEqual(kCTFontMiniSystemFontType,  6)
-        self.assertEqual(kCTFontMiniEmphasizedSystemFontType,  7)
-        self.assertEqual(kCTFontViewsFontType,  8)
-        self.assertEqual(kCTFontApplicationFontType,  9)
+        self.assertEqual(kCTFontNoFontType, 4294967295) # (uint32_t)-1
+        self.assertEqual(kCTFontUserFontType, 0)
+        self.assertEqual(kCTFontUserFixedPitchFontType, 1)
+        self.assertEqual(kCTFontSystemFontType, 2)
+        self.assertEqual(kCTFontEmphasizedSystemFontType, 3)
+        self.assertEqual(kCTFontSmallSystemFontType, 4)
+        self.assertEqual(kCTFontSmallEmphasizedSystemFontType, 5)
+        self.assertEqual(kCTFontMiniSystemFontType, 6)
+        self.assertEqual(kCTFontMiniEmphasizedSystemFontType, 7)
+        self.assertEqual(kCTFontViewsFontType, 8)
+        self.assertEqual(kCTFontApplicationFontType, 9)
         self.assertEqual(kCTFontLabelFontType, 10)
         self.assertEqual(kCTFontMenuTitleFontType, 11)
         self.assertEqual(kCTFontMenuItemFontType, 12)
@@ -104,6 +121,12 @@ class TestCTFont (TestCase):
 
 
         self.assertEqual(kCTFontTableBASE, fourcc(b'BASE'))
+        self.assertEqual(kCTFontTableCBDT, fourcc(b'CBDT'))
+        self.assertEqual(kCTFontTableCBLC, fourcc(b'CBLC'))
+        self.assertEqual(kCTFontTableCFF, fourcc(b'CFF '))
+        self.assertEqual(kCTFontTableCFF2, fourcc(b'CFF2'))
+        self.assertEqual(kCTFontTableCOLR, fourcc(b'COLR'))
+        self.assertEqual(kCTFontTableCPAL, fourcc(b'CPAL'))
         self.assertEqual(kCTFontTableCFF, fourcc(b'CFF '))
         self.assertEqual(kCTFontTableDSIG, fourcc(b'DSIG'))
         self.assertEqual(kCTFontTableEBDT, fourcc(b'EBDT'))
@@ -112,25 +135,34 @@ class TestCTFont (TestCase):
         self.assertEqual(kCTFontTableGDEF, fourcc(b'GDEF'))
         self.assertEqual(kCTFontTableGPOS, fourcc(b'GPOS'))
         self.assertEqual(kCTFontTableGSUB, fourcc(b'GSUB'))
+        self.assertEqual(kCTFontTableHVAR, fourcc(b'HVAR'))
         self.assertEqual(kCTFontTableJSTF, fourcc(b'JSTF'))
         self.assertEqual(kCTFontTableLTSH, fourcc(b'LTSH'))
+        self.assertEqual(kCTFontTableMATH, fourcc(b'MATH'))
+        self.assertEqual(kCTFontTableMVAR, fourcc(b'MVAR'))
         self.assertEqual(kCTFontTableOS2, fourcc(b'OS/2'))
         self.assertEqual(kCTFontTablePCLT, fourcc(b'PCLT'))
+        self.assertEqual(kCTFontTableSTAT, fourcc(b'STAT'))
+        self.assertEqual(kCTFontTableSVG, fourcc(b'SVG '))
         self.assertEqual(kCTFontTableVDMX, fourcc(b'VDMX'))
         self.assertEqual(kCTFontTableVORG, fourcc(b'VORG'))
+        self.assertEqual(kCTFontTableVVAR, fourcc(b'VVAR'))
         self.assertEqual(kCTFontTableZapf, fourcc(b'Zapf'))
         self.assertEqual(kCTFontTableAcnt, fourcc(b'acnt'))
+        self.assertEqual(kCTFontTableAnkr, fourcc(b'ankr'))
         self.assertEqual(kCTFontTableAvar, fourcc(b'avar'))
         self.assertEqual(kCTFontTableBdat, fourcc(b'bdat'))
         self.assertEqual(kCTFontTableBhed, fourcc(b'bhed'))
         self.assertEqual(kCTFontTableBloc, fourcc(b'bloc'))
         self.assertEqual(kCTFontTableBsln, fourcc(b'bsln'))
+        self.assertEqual(kCTFontTableCidg, fourcc(b'cidg'))
         self.assertEqual(kCTFontTableCmap, fourcc(b'cmap'))
         self.assertEqual(kCTFontTableCvar, fourcc(b'cvar'))
         self.assertEqual(kCTFontTableCvt, fourcc(b'cvt '))
         self.assertEqual(kCTFontTableFdsc, fourcc(b'fdsc'))
         self.assertEqual(kCTFontTableFeat, fourcc(b'feat'))
         self.assertEqual(kCTFontTableFmtx, fourcc(b'fmtx'))
+        self.assertEqual(kCTFontTableFond, fourcc(b'fond'))
         self.assertEqual(kCTFontTableFpgm, fourcc(b'fpgm'))
         self.assertEqual(kCTFontTableFvar, fourcc(b'fvar'))
         self.assertEqual(kCTFontTableGasp, fourcc(b'gasp'))
@@ -143,9 +175,12 @@ class TestCTFont (TestCase):
         self.assertEqual(kCTFontTableHsty, fourcc(b'hsty'))
         self.assertEqual(kCTFontTableJust, fourcc(b'just'))
         self.assertEqual(kCTFontTableKern, fourcc(b'kern'))
+        self.assertEqual(kCTFontTableKerx, fourcc(b'kerx'))
         self.assertEqual(kCTFontTableLcar, fourcc(b'lcar'))
+        self.assertEqual(kCTFontTableLtag, fourcc(b'ltag'))
         self.assertEqual(kCTFontTableLoca, fourcc(b'loca'))
         self.assertEqual(kCTFontTableMaxp, fourcc(b'maxp'))
+        self.assertEqual(kCTFontTableMeta, fourcc(b'meta'))
         self.assertEqual(kCTFontTableMort, fourcc(b'mort'))
         self.assertEqual(kCTFontTableMorx, fourcc(b'morx'))
         self.assertEqual(kCTFontTableName, fourcc(b'name'))
@@ -153,9 +188,11 @@ class TestCTFont (TestCase):
         self.assertEqual(kCTFontTablePost, fourcc(b'post'))
         self.assertEqual(kCTFontTablePrep, fourcc(b'prep'))
         self.assertEqual(kCTFontTableProp, fourcc(b'prop'))
+        self.assertEqual(kCTFontTableSbit, fourcc(b'sbit'))
         self.assertEqual(kCTFontTableTrak, fourcc(b'trak'))
         self.assertEqual(kCTFontTableVhea, fourcc(b'vhea'))
         self.assertEqual(kCTFontTableVmtx, fourcc(b'vmtx'))
+        self.assertEqual(kCTFontTableXref, fourcc(b'xref'))
 
         self.assertEqual(kCTFontTableOptionNoOptions, 0)
         self.assertEqual(kCTFontTableOptionExcludeSynthetic, (1 << 0))
@@ -379,6 +416,12 @@ class TestCTFont (TestCase):
         v = CTFontGetTypeID()
         self.assertIsInstance(v, (int, long))
 
+        v, a = CTFontGetLigatureCaretPositions(font, gl[0], None, 5)
+        self.assertIsInstance(v, int)
+        self.assertIsInstance(a, tuple)
+        self.assertEqual(len(a), 5)
+        self.assertTrue(all(isinstance(x, float) for x in a))
+
     @min_os_level('10.6')
     def testFunctions10_6(self):
         self.assertResultIsCFRetained(CTFontCreateWithNameAndOptions)
@@ -386,26 +429,60 @@ class TestCTFont (TestCase):
         self.assertIsInstance(v, CTFontRef)
 
 
+    @expectedFailureIf(os_release() == '10.6')
+    @min_os_level('10.6')
+    def testFunctions10_6_crash(self):
         descr = CTFontDescriptorCreateWithNameAndSize(
                 b"Courier".decode('latin1'), 14.0)
         self.assertNotEqual(descr, None)
 
-        # FIXME: this crashes the interpreter, without a clear reason
-        return
-
         self.assertResultIsCFRetained(CTFontCreateWithFontDescriptorAndOptions)
+
+        # FIXME: this crashes the interpreter, without a clear reason
+        if os_release() == '10.6':
+            self.fail("hard crash in test for CTFontCreateWithFontDescriptorAndOptions")
         v = CTFontCreateWithFontDescriptorAndOptions(descr, 14.0, None, 0)
         self.assertIsInstance(v, CTFontRef)
 
-    @expectedFailure
     @min_os_level('10.7')
     def testFunctions10_7(self):
-        self.fail("CTFontDrawGlyphs")
+        self.assertArgHasType(CTFontDrawGlyphs, 0, b"^{__CTFont=}")
+        self.assertArgHasType(CTFontDrawGlyphs, 1, b"n^S")
+        self.assertArgSizeInArg(CTFontDrawGlyphs, 1, 3)
+        self.assertArgHasType(CTFontDrawGlyphs, 2, b"n^" + CGPoint.__typestr__)
+        self.assertArgSizeInArg(CTFontDrawGlyphs, 2, 3)
+        self.assertArgHasType(CTFontDrawGlyphs, 3, objc._C_NSUInteger)
+        self.assertArgHasType(CTFontDrawGlyphs, 4, b"^{CGContext=}")
+
 
-    @expectedFailure
     @min_os_level('10.8')
     def testFunctions10_8(self):
-        self.fail("CTFontGetOpticalBoundsForGlyps")
+
+        font = CTFontCreateUIFontForLanguage(
+                kCTFontMiniSystemFontType,
+                10.0, "nl_NL")
+        self.assertTrue(font is not None)
+        v, gl = CTFontGetGlyphsForCharacters(font,
+                b"hello".decode('latin1'), None, 5)
+        self.assertTrue(v)
+        self.assertIsInstance(gl, tuple)
+
+        v, r = CTFontGetOpticalBoundsForGlyphs(font, gl, None, 5, 0)
+        self.assertIsInstance(v, CGRect)
+        self.assertIsInstance(r, tuple)
+        self.assertEqual(len(r), 5)
+        self.assertIsInstance(r[0], CGRect)
+
+        self.assertResultIsCFRetained(CTFontCopyDefaultCascadeListForLanguages)
+
+    @min_os_level('10.10')
+    def testConstants10_10(self):
+        self.assertIsInstance(kCTFontOpenTypeFeatureTag, unicode)
+        self.assertIsInstance(kCTFontOpenTypeFeatureValue, unicode)
+
+    @min_os_level('10.13')
+    def testConstants10_13(self):
+        self.assertIsInstance(kCTFontVariationAxisHiddenKey, unicode)
 
 if __name__ == "__main__":
     main()
